
<div id="app">
    <div style="margin-bottom: 10px;background-image: linear-gradient(to right, #0acffe 0%, #495aff 100%);width: 80%;height: 100px;border-top-left-radius: 10px;border-top-right-radius: 10px"">
        <div style="line-height: 100px;font-size: 18px;font-weight: bold;color: #fff">
            {{content_title}}
        </div>
        <div style="background-image: linear-gradient(-225deg, #E3FDF5 0%, #FFE6FA 100%);width: 100%;border-bottom-left-radius: 10px;border-bottom-right-radius: 10px">
            <table width="100%" cellpadding="1" cellspacing="1" style="font-size:18px;margin:0;">

                <tbody>

                <tr >
                    <td style="padding:0 0 20px; width: 25%;">
                        标题:
                    </td>
                    <td style="padding:0 0 20px; width: 25%;">
                        {{ mail_title }}
                    </td>
                </tr>
                <tr>

                    <td style="padding:0 0 20px; width: 25%;">
                        总用例数:
                    </td>
                    <td style="padding:0 0 20px; width: 25%;">
                        {{ count }}
                    </td>


                    <td style="padding:0 0 20px; width: 25%;">
                        测试人员:
                    </td>
                    <td style="padding:0 0 20px; width: 25%;">
                        {{ tester }}
                    </td>
                </tr>
                <tr>
                    <td style="padding:0 0 20px; width: 25%;">
                        开始时间:
                    </td>
                    <td style="padding:0 0 20px; width: 25%;">
                        {{ case_run_startTime }}
                    </td>
                    <td style="padding:0 0 20px; width: 25%;">
                        结束时间:
                    </td>
                    <td style="padding:0 0 20px; width: 25%;">
                        {{ case_run_endTime }}
                    </td>
                </tr>
                <tr>
                    <td style="padding:0 0 20px; width: 25%;" >
                        运行时长:
                    </td>
                    <td style="padding:0 0 20px; width: 25%;">
                        {{ case_run_time }}
                    </td>
                    <td style="padding:0 0 20px; width: 25%;"></td>
                    <td style="padding:0 0 20px; width: 25%;"></td>
                </tr>


                    <td style="padding:0 0 20px; width: 25%;">
                        通过用例:
                    </td>
                    <td style="padding:0 0 20px; width: 25%;">
                        {{ pass_count }}
                    </td>
                    <td style="padding:0 0 20px; width: 25%;">
                        通过率:
                    </td>
                    <td style="padding:0 0 20px; width: 25%;">
                        {{ pass_rate }}
                    </td>
                </tr>

                <tr>
                    <td style="padding:0 0 20px; width: 25%;">
                        失败用例:
                    </td>
                    <td style="padding:0 0 20px; width: 25%;">
                        {{ fail_count }}
                    </td>
                    <td style="padding:0 0 20px; width: 25%;">
                        失败率:
                    </td>
                    <td style="padding:0 0 20px; width: 25%;">
                        {{ failure_rate }}
                    </td>
                </tr>
                <tr>
                    <td style="padding:0 0 20px; width: 25%;">
                        错误用例:
                    </td>
                    <td style="padding:0 0 20px; width: 25%;">
                        {{ error_count }}
                    </td>
                    <td style="padding:0 0 20px; width: 25%;">
                        错误率:
                    </td>
                    <td style="padding:0 0 20px; width: 25%;">
                        {{ error_rate }}
                    </td>
                </tr>
                <tr>
                    <td style="padding:0 0 20px; width: 25%;">
                        跳过用例:
                    </td>
                    <td style="padding:0 0 20px; width: 25%;">
                        {{ skip_count }}
                    </td>
                    <td style="padding:0 0 20px; width: 25%;">
                        跳过率:
                    </td>
                    <td style="padding:0 0 20px; width: 25%;">
                        {{ skip_rate }}
                    </td>
                </tr>
                </tbody>


            </table>
        </div>
    </div>
</div>


