#!/usr/bin/env python
#-*- coding:utf-8 -*-

# authors:guanfl
# 2022/10/21
import os
import shelve
s = shelve.open('test_shelf.db')         #
#
# s['小明'] = {'int': 10, 'float': 9.5, 'String': 'Sample data'}
# s['MM'] = [1, 2, 4,"data"]
#
# print(s["小明"])
# s.close()

os.remove("./")
